#!/usr/bin/env python
#
# MB_ISDNmonp_cc.py Version 0.4.1
# What it does: 
#	python version of client part for 
#	MB_ISDNmonp_multiclient_server.tcl (old) or 
#	MB_ISDNmonp_svr.py (new).
#	the server part distributes information about in/outbound
#	isdn-connections from /dev/isdninfo.
#	this client program retrieves this information and runs
#	a script on opening/closing connections
#	there are also older versions of this written in TCL:
#	MB_ISDNmonp_consoleclient.tcl for running from console/inittab
#	MB_ISDNmonp_client.tcl for running from X (uses TK/ wishx)
#	In the future i might also write an graphical version of this
#	with TkInter or PyGTK or maybe both.
#	I'm even thinking about writing a CGI version of this.
#
#
#
# Copyright (c) 2000 by Manuel Bessler <m.bessler@gmx.net>
# 
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import sys, posix, string, time, os
from socket import *


BUFSIZE = 1024
CR="\r"
LF="\n"

online_action_out="/root/.MB_ISDNmonp_client.online_action"
offline_action=None

PORT = 800
HOSTNAME="192.168.0.9"

def gettime():
	localtime=time.localtime(time.time())
	return time.strftime("%d.%m.%Y %H:%M:%S",localtime)


def miscerror():
	datetime=gettime()
	sys.stderr.write("%s ERROR Internal Error\r\n%s ERROR Please restart program\r\n%s ERROR If this does not help, check the server side\r\n" % (datetime, datetime, datetime))
	sys.exit(1)


def signal_online():
    print "%s Gone ONline!" % gettime()
    if online_action_out:
    	ret=os.system(online_action_out)
	if ret:
	    sys.stderr.write("%s Error executing online_action_out: %s\r\n" % (gettime(), online_action_out))
	else:
	    return 0

    
def signal_offline():
    print "%s Gone OFFline!" % gettime()
    if offline_action:
	ret=os.system(offline_action)
	if ret:
	    sys.stderr.write("%s Error executing offline_action: %s\r\n" % (gettime(), offline_action))
	else:
    	    return 0

# smooth exit to prevent rapid respawning from inittab
#
def slow_exit():
    sys.stderr.write("%s ERROR socket operation failed.\r\n" % gettime())
    sys.stderr.write("%s Exitting in 25 secs.\r\n" % gettime())
    time.sleep(25)
    sys.exit(1)

# Main program (called at the end of this file).
#
def main():
	hostname = HOSTNAME
	control(hostname)


# Control process (user interface and user protocol interpreter).
#
def control(hostname):
	chcount=0
	call_done=0
	chcount_done=0
	oldcolor=["darkseagreen", "darkseagreen"]
	newcolor=["darkseagreen", "darkseagreen"]
	callstat=["darkseagreen", "darkseagreen"]
	calltype=["initialising", "initialising"]
	callnr=["initialising", "initialising"]

	#
	# Create control connection
	#
	connected=0
	while not connected:
	    try:
    		s = socket(AF_INET, SOCK_STREAM)
		s.connect(hostname, PORT)
	    except error, msg:
		sys.stderr.write("%s ERROR Open socket failed:\r\n%s REASON: %s, ErrCode=%s\r\n%s Is the server running?\r\n" % (gettime(), gettime(), str(msg[1]), str(msg[0]), gettime()))
		sys.stderr.write("%s Retrying in 25 secs...\r\n" % gettime())
		time.sleep(25)	# sleep 25secs to prevent respawning from inittab to be delayed
	    else:
		connected=1
		print "Connection established"

	try:
	    f = s.makefile('r') # Reading the replies is easier from a file...
	except:
	    slow_exit()
	#
	# Control loop
	#
	try:
	    s.send("LIST\r\n")
	except:
	    slow_exit()
	while 1:
		try:
		    line, code = getreply(f)
		except:			# if the server dies at this point, the former cmd raises a ValueError
		    slow_exit()
		if not line: break # Really an error
		elif code in ('221', 'EOF'): break
		elif code == "220": continue    #just the greeting msg
		elif code == '510':		#/dev/isdninfo can't be read, retry in 25 secs
		    sys.stderr.write("%s ERROR: Server can't read /dev/isdninfo\r\n" % gettime())
		    sys.stderr.write("%s Retrying in 5 secs...\r\n" % gettime())
		    time.sleep(5)
		elif code == '210':
    		    if line[3:4] == '-':
			    if line[:6] == "210-Se":
				chcount=0
				del callnr[:]
				del callstat[:]
				del calltype[:]
				call_done=0
			    if line[4:5] == "X":
				chcount=chcount+1
				ct=line[5:]
				if ct=="":
				    ct="working..."
				calltype.append(ct)
			    if line[4:5] == "Y":
				cn=line[5:]
				if cn=="":
				    cn="working..."
				callnr.append(cn)
			    if line[4:5] == "Z":
				cs=line[5:]
				if cs=="":
				    cs="working..."
				callstat.append(cs)
			    continue
		    if line[:3] == code and line[3:4] != '-': 
			call_done=1
			for i in range(int(chcount)):
			    newcolor[i]=callstat[i]
			    if oldcolor[i] != newcolor[i]:
				if newcolor[i] == "red":
				    signal_online()
				if newcolor[i] == "darkseagreen":
				    signal_offline()
#0			    print "Line %s: %s Status: %s Number: %s" % (str(i), str(calltype[i]), str(callstat[i]), str(callnr[i]))
			    oldcolor[i]=newcolor[i]
		else:
		    sys.stderr.write("%s ERROR: Unrecognized: %s\r\n" % (gettime(), str(line)))
		    sys.stderr.write("%s Retrying in 25 secs...\r\n" % gettime())
		    time.sleep(25)
		try:
		    time.sleep(2)
    		    s.send("LIST\r\n")
		except:
		    slow_exit()

# Process an ftp reply and return the 3-digit reply code (as a string).
# The reply should be a line of text starting with a 3-digit number.
# If the 4th char is '-', it is a multi-line reply and is
# terminate by a line starting with the same 3-digit number.
# Any text while receiving the reply is echoed to the file.
#
def getreply(f):
	try:
	    line = f.readline()
	except:
	    slow_exit()
	if not line: return 'EOF'
	if line[-1:] == LF:
	    line=line[:-1]
	if line[-1:] == CR:
	    line=line[:-1]
	code = line[:3]
	return line, code


# Call the main program.
#
main()


# make emacs behave:
#
# Local variables:
# py-indent-offset: 4
# py-block-comment-prefix: "##"
# tab-width: 4
# End:
#