#!/usr/bin/env python
#
# MB_ISDNmonp_svr.py Version 0.4.1
# What it does:
#	It lets you poll information about /dev/isdninfo of
#	the system it is running on. It has a very simple,
#	ftp-like Protocol. This is a complete rewrite of
#	MB_ISDNmonp_multiclient_server.tcl which itself is
#	based on ISDNmonp which is based on ISDNmon ...
#	There are also old clients in tcl available:
#	MB_ISDNmonp_consoleclient.tcl for running from console/inittab
#	and
#	MB_ISDNmonp_client.tcl for running from X with Tk (wishx)
#	And this is a new client written in Python for console/inittab:
#	MB_ISDNmonp_cc.py
#	In the future i might write an graphical frontend for the python
#	version as well (TkInter/PyGTK or maybe both) and possible a 
#	small CGI.
#
#
# Copyright (c) 2000 by Manuel Bessler <m.bessler@gmx.net>
# 
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import string, sys, os, time, thread

from socket import *

DEBUG=0
DEBUG2=0


PORT=800

INFODEV="/dev/isdninfo"
INFODEVFP=None
BUFSIZE=1024
USAGE=('None', 'Raw', 'Modem', 'Net', 'Voice', 'Fax')

def gettime():
	localtime=time.localtime(time.time())
	return time.strftime("%d.%m.%Y %H:%M:%S",localtime)

def printerr(msg):
    sys.stdout.write(msg)


# smooth exit to prevent rapid respawning from inittab
#
def slow_exit():
    printerr("%s ERROR: socket operation failed" % gettime())
    printerr("%s Exitting in 25 secs" % gettime())
    time.sleep(25)
    sys.exit(1)


#this is what my /dev/isdninfo looks when offline:
# note that the character behind the colon is a tab
# idmap:	HiSax HiSax - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# chmap:	0 1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 
# drmap:	0 0 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 
# usage:	0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 
# flags:	0 ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? ? 
# phone:	??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? 

def getinfo():
    try:
	f=open(INFODEV, "r")
    except:
	printerr("%s Error: Could not open IsdnInfoDevice %s\r\n" % (gettime(), INFODEV))
	havetoclose=0
	havetoread=0
	return None
    else:
	isdnstatus={}
	isdnstatus['idmap'] = string.split( f.readline() )
	isdnstatus['chmap'] = string.split( f.readline() )
	isdnstatus['drmap'] = string.split( f.readline() )
	isdnstatus['usage'] = string.split( f.readline() )
	isdnstatus['flags'] = string.split( f.readline() )
	isdnstatus['phone'] = string.split( f.readline() )
	if DEBUG2==1:
	    print isdnstatus
	f.close()
	return isdnstatus


def get_STATUS(sock):
    isdnstatus=getinfo()
    if not isdnstatus:
	return None

    dcount=0
    emptycount=0
    ret=[]
    
    flist=isdnstatus['flags'][1:]
    ulist=isdnstatus['usage'][1:]
    ilist=isdnstatus['chmap'][1:]
    rlist=isdnstatus['phone'][1:]

    for i in flist:
	if i == "?":
	    emptycount = emptycount + 1
    dcount=len(flist)-emptycount

    if DEBUG2==1:
	print "assert1",
	print "dcount=",
	print dcount
    usg=[]
    rem=[]
    onlc=[]

    for i in range(dcount):
        if DEBUG2==1:
	    print "assert2"
	chcount=0
	chi=0

	usg.append([])
	rem.append([])
	onlc.append([])
	clist=isdnstatus['drmap'][1:]
	for j in clist:
#	    if DEBUG2==1:
#		print "assert3: len=%d j=%s ; clist:%s" % (len(clist), str(j), str(clist))
	    usg[i].append([])
	    rem[i].append([])
	    onlc[i].append([])
    	    if j == str(i):
	        if DEBUG2==1:
		    print "assert4",
		    print "j=",
		    print j,
		    print "i=",
		    print i
		uplain   = int(ulist[chi]) & 7
		outgoing = int(ulist[chi]) & 128
	        if DEBUG2==1:
		    print "assert5",
		    print chcount,
		    print uplain,
		    print USAGE[uplain]
		usg[i][chcount] = USAGE[uplain]
		if DEBUG2==1:
		    print "usg: ",usg
		rem[i][chcount] = rlist[chi] 
		if int(flist[i]) & 1<<chcount:
#		    if not onl[i][chcount][:2] == "No":
#			signal_online(outgoing, rem[i][chcount])
#		    onl[i][chcount]="On"
		    if outgoing:
			onlc[i][chcount]="red"
		    else:
			onlc[i][chcount]="yellow"
		else:
#		    if onl[i][chcount][:3] == "Yes":
#			signal_offline()
#		    onl[i][chcount]="Off"
		    onlc[i][chcount]="darkseagreen"
		chcount=chcount+1
	    chi=chi+1
	for i2 in range(chcount):
	    ret.append("210-X%s\n210-Y%s\n210-Z%s\r\n" % (usg[i][i2], rem[i][i2], onlc[i][i2]))
    return ret



#-----------------------------------------------------------------------------
# upper part: retrieving info from /dev/isdninfo
#
# lower part: server part
#-----------------------------------------------------------------------------

# Implement the service
def doservice(sock):
    while 1:
    	try:
	    l=sock.recv(BUFSIZE)    	# get the client packet
	except error, msg:
	    printerr("%s Peer closed prematurely\r\n" % gettime())
	    sock.close()
	    break
        if not l:			# client gone or finished
    	    try:
		sock.close()        	# release the servers client channel
		break
	    except:
		pass
	if DEBUG2==1:
	    print "#%s#" % l[:-2]	# cut the trailing "\r\n"
        l=string.upper(l[:-2])
	if l == "QUIT":
	    try:
		sock.send("221 Goodbye.\r\n")
    		sock.close()
		break
	    except:
		pass
		
#CHAN is no longer supported, instead get the channelcount form LIST
#        elif l == "CHAN":
#	    sock.send("216-Sending channelcount.\r\n")
#	    sock.send("216-+%s\r\n" % str(chcount))
#	    sock.send("216 End of channelcount.\r\n")
	
#STAT is no longer supported, use LIST instead:
        elif l == "LIST":
	    status=get_STATUS(sock)
	    if not status:
		try:
		    sock.send("510-Error: Could not read from IsdnInfoDevice\r\n")
		    sock.send("510 Please contact Site Administrator\r\n")
		except:
		    pass
	    else:
		try:
		    sock.send("210-Sending list.\r\n")
		    for i in status:
			sock.send(i)
		    sock.send("210 End of LIST.\r\n")
		except:
		    pass

	
	elif l == "HELP":
	    try:
    		sock.send("214-List of available commands.\r\n")
		sock.send("214-HELP       Get this Help\r\n")
		sock.send("214-LIST       List Info about ISDN-Channels\r\n")
		sock.send("214-QUIT       Close Connection\r\n")
    		sock.send("214-The following Commands are no longer supported:\r\n")
    		sock.send("214-STAT => use LIST instead\r\n")
    		sock.send("214-CHAN => make your client count the Channels tru LIST\r\n")	    
    		sock.send("214-NOTE: There may be vendor spezific Extensions that are not listed here\r\n")
		sock.send("214 Direct Comments to m.bessler@gmx.net.\r\n")
	    except:
	        pass

	
	else:
	    try:
		sock.send("500 %s: Command not recognized. Try HELP.\r\n" % l)
	    except:
	        pass
	




# Accept-Connection handler for Server. 
# called When client makes a connection to the server
# Its passed the channel we're to communicate with the client on, 
# The address of the client and the port we're using
#
# Setup a handler for (incoming) communication on 
# the client channel - send connection Reply and log connection
def new_client(sock, addr, port):
    print "%s Connect from %s:%s" % (gettime(), str(addr), str(port))
    try:
	sock.send("220 ISDNInfo-Server Version 0.4.0 ready. Welcome %s:%s.\r\n" % (str(addr), str(port)))
	doservice(sock)
    except:
	pass
    try:
        sock.close()
    except:
	pass
    print "%s Disconnected %s:%s" % (gettime(), str(addr), str(port))    
    raise SystemExit		# End thread



def main():
    s=socket(AF_INET, SOCK_STREAM)
    try:
	s.bind('',PORT)
	s.listen(10)			# how many unaccepted connections may exist before refusing new connections 
    except error, msg:
	printerr("%s ERROR %s: Reason: %s" % (gettime(), str(msg[0]), msg[1]))
	slow_exit()
    print "%s Server Ready, Interface bound." % gettime()
    while 1:
	try:
	    sock, (addr, port) = s.accept()
	    thread.start_new_thread(new_client, (sock, addr, port))
	except:
	    try:
		s.close()
	    except:
		pass
	    slow_exit()

    
if __name__ == "__main__": main()


#########################
# make emacs behave:
#
# Local variables:
# py-indent-offset: 4
# py-block-comment-prefix: "##"
# tab-width: 4
# End:
#