#!/bin/sh
# the next line restarts using wishx \
exec wishx "$0" "$@"

# this is MB_ISDNmonp_client.tcl
# Modifications Copyright 1999 by Manuel Bessler
# modified by MB to support client server
# for the server part look for MB_ISDNmonp_server.tcl
#


# this is isdnmonp - "p" stands for patched (or professional? ;-)) version
# of 'isdnmon'
#
# changes:
# - some unused widgets deleted, so you have a very little window
# - use of 'wishx' instaed of 'wish'
# - use of 'select' to detect id the status of /dev/isdn* is changed,
#   so you don't need an extra cpu for this tool ;-)


# You can configure the actions to be performed when a channel
# changes it's state. Each action is simply execution of an
# external command. The online-action-strings can contain a '%s'
# which will be replaced by the remote Phonenumber.
# The following definitions are just Samples
#set online_action_out {rsh wuemaus "dialsound %s &"}
#set offline_action    {rsh wuemaus "splay -q /sounds/hangup &"}

# Uncomment these to disable actions.
#set online_action_out "/root/.MB_ISDNmonp_client.online_action"
set online_action_out ""
set offline_action    ""

set infohost 192.168.0.9
set infoport 800

set usage {None Raw Modem Net Voice Fax}

proc chan {} {
  global chcount sock 
  puts $sock "CHAN" 
}

proc xlist {} {
  global sock 
  puts $sock "LIST" 
}

proc quit {} {
  global sock 
  puts $sock "QUIT" 
}

proc stat {} {
  global sock 
  puts $sock "STAT" 
}

proc isdnerror {} {
  wm withdraw .
  tk_dialog .err Error "No ISDN-infodevice found" error 0 "Exit"
  exit
}

proc miscerror {} {
  wm withdraw .
  tk_dialog .err Error "Internal Error\Please restart program\If this doesnt help, check the server side" error 0 "Exit"
  exit
}

proc get_chcount {str} {
  global chcount chcount_done
  if {$str == "216-Sending channelcount."} then {
	set chcount_done 0
	return
  }
  if {$str == "216 End of channelcount."} then {
	set chcount_done 1
	return
  }
  if {[string index $str 4] == "+"} then {
	set chcount [string range $str 5 end]
  }
}


proc get_list {str} {
  global chcount callnr calltype callstat call_done
  if {$str == "210-Sending list."} then {
	  set chcount 0
	  set callnr ""
	  set calltype ""
	  set callstat ""
	  set call_done 0
  }
  if {$str == "210 End of LIST."} then {
	set call_done 1
	return
  }
  if {[string index $str 4] == "X"} then {
	incr chcount
	set ct [string range $str 5 end]
	if {$ct == ""} then {set ct "working..."}
	lappend calltype $ct
  }
  if {[string index $str 4] == "Y"} then {
	set cn [string range $str 5 end]
	if {$cn == ""} then {set cn "working..."}
	lappend callnr $cn
  }
  if {[string index $str 4] == "Z"} then {
	set cs [string range $str 5 end]
	if {$cs == ""} then {set cs "working..."}
	lappend callstat $cs
  }
}

proc read_sock {sock} {
  set l [gets $sock]
  if {$l != ""} then {
     switch -regexp $l {
	^5[.]*	{#error
puts "MB_ISDNmonp_client.tcl: error 5??,\n server returned: $l"
		miscerror
		}
	^210[.]*	{#list
		get_list $l
		}
	^216[.]*	{#chcount
	 	get_chcount $l
		}
	^211[.]*	{#status
	 	return
		}
	^214[.]*	{#help
		return
		}
	^220[.]*	{return }
	^221[.]*	{#goodbye
		 exit}
	^240[.]*	{# isdnerror
		isdnerror}
	default {#unkown command
puts "MB_ISDNmonp_client.tcl: server returned: $l"
		miscerror
		}
     }	
  }
}

proc init {} {
	global sock infohost infoport

	set sock [socket $infohost $infoport]
	fileevent $sock readable "read_sock $sock"
	fconfigure $sock -buffering line
}

proc getinfo {} {
  global chcount callnr calltype callstat chcount_done call_done
  set chcount_done 0
  chan
  while {$chcount_done==0} {vwait chcount_done}
  set call_done 0
  xlist 
  while {$call_done==0} {vwait call_done}
#puts "getinfo: callxxx: $callnr -  $calltype - $callstat"
}


proc signal_online {} {
  global online_action_out semaphore1
  if {$semaphore1 == 1} then {return}
  set time [clock seconds]
  set datetime [clock format $time -format "%c"]
puts "$datetime Gone ONline!"
 if {![string length $online_action_out]} {
    return
  }
  set semaphore1 1
  catch {eval exec $online_action_out }
}

proc signal_offline {} {
  global offline_action semaphore1
  set semaphore1 0
  set time [clock seconds]
  set datetime [clock format $time -format "%c"]
puts "$datetime Gone OFFline!"
 if {![string length $offline_action]} {
    return
  }
  catch {eval exec $offline_action }
}




proc doloop {} {
  global chcount callnr calltype callstat
  getinfo
#puts "entered doloop"
  for {set i 0} {$i < [expr $chcount - 0]} {incr i} {
      .did.st$i.usg configure -text [lindex $calltype $i]
# retrieve old color
      set oldcolor [.did.st$i.rem cget -background]
      set newcolor [lindex $callstat $i]
      .did.st$i.rem configure -text [lindex $callnr $i] \
	       -background $newcolor
      update idletasks
      if {$oldcolor != $newcolor && $newcolor=="red"} then {signal_online}
      if {$oldcolor != $newcolor && $newcolor=="darkseagreen"} then {signal_offline}
  }
  after 1000 doloop
}


proc main {} {
global chcount callnr calltype callstat semaphore1
set semaphore1 0
set chcount 0
set callnr "initialising..."
set calltype "initialising..."
set callstat darkseagreen
update idletasks
after 500
update idletasks
init
getinfo
frame .did -relief ridge -borderwidth 2
for {set i 0} {$i < [expr $chcount - 0]} {incr i} {
	frame .did.st$i
	label .did.st$i.l3 -text " Use: " -width 5 -anchor w
	label .did.st$i.rem -text [lindex $callnr $i] -width 13 -anchor w \
	  -relief ridge -background [lindex $callstat $i]
	label .did.st$i.usg -width 6 -anchor w -relief ridge \
	  -text [lindex $calltype $i] 
	pack .did.st$i.rem -side left
	pack .did.st$i.usg .did.st$i.l3 -side right
	pack .did.st$i -side top -padx 3 -pady 3 -fill x
}
pack .did -fill x -padx 3 -pady 3
doloop
}

main






