#!/bin/sh
# the next line restarts using tclx \
exec tclx "$0" "$@"

set svcPort 9999

global active
set active 0

set infodev /dev/isdninfo
set infodevfp ""
set infoexit 0

set usage {None Raw Modem Net Voice Fax}

proc init {} {
	global infodev infodevfp

	if [catch {set infodevfp [open $infodev r]}] {
			isdnerror 
		}
	close $infodevfp
	set infodevfp ""
	
}

proc isdnerror {} {
  global active gsock
  puts $gsock "240 Error: No ISDN-infodevice found"
}

proc getinfo {} {
  global infodev infodevfp is withselect
  

	set f [open $infodev r]
	set havetoclose 1
	set havetoread 1
	
	if {$havetoread} {
		gets $f is(idmap)
		gets $f is(chmap)
		gets $f is(drmap)
		gets $f is(usage)
		gets $f is(flags)
		gets $f is(phone)
	}
	if {$havetoclose} {
		close $f
	}
}

proc doloop {} {
  global infoexit is usage yesno onl
  global active gsock chcount
 
  if {$infoexit} {
  }
  getinfo
  set flist [lreplace $is(flags) 0 0]
  set ulist [lreplace $is(usage) 0 0]
  set ilist [lreplace $is(chmap) 0 0]
  set rlist [lreplace $is(phone) 0 0]
  set dcount 0
  foreach n $flist {
    if [string compare $n ?] {
      incr dcount
    }
  }
  for {set i 0} {$i<$dcount} {incr i} {
    set chcount 0
    set chi 0
    set clist [lreplace $is(drmap) 0 0]
    foreach n $clist {
      if {![string compare $n $i]} {
	set uplain [expr [lindex $ulist $chi]&7]
	set outgoing [expr [lindex $ulist $chi]&128]
	set usg($i,$chcount) [lindex $usage $uplain]
	set rem($i,$chcount) [lindex $rlist $chi]
	if [expr [lindex $flist $i] & (1<<$chcount)] {
	  if {![string compare $onl($i,$chcount) No]} {
	    signal_online $outgoing $rem($i,$chcount)
	  }
	  set onl($i,$chcount) "On"
	  if {$outgoing} {
	    set onlc($i,$chcount) red
          } else {
	    set onlc($i,$chcount) yellow
          }
	} else {
	  if {![string compare $onl($i,$chcount) Yes]} {
	    signal_offline
	  }
	  set onl($i,$chcount) "Off"
	  set onlc($i,$chcount) darkseagreen
	}
	incr chcount
      }
      incr chi
    }
    for {set j 0} {$j<$chcount} {incr j} {
      puts $gsock "210-X$usg($i,$j)\n210-Y$rem($i,$j)\n210-Z$onlc($i,$j)"
    }
  }
}

proc main {} {
  global infodev infodevfp is withselect
  global infoexit is usage yesno onl
  global active gsock chcount
  init
  getinfo
  if [info exists is(flags)] {
    set flist [lreplace $is(flags) 0 0]
    set ulist [lreplace $is(usage) 0 0]
    set rlist [lreplace $is(phone) 0 0]
    set dcount 0
    foreach n $flist {
      if [string compare $n ?] {
        incr dcount
      }
    }
    if {$dcount==0} {
    } else {
      for {set i 0} {$i<$dcount} {incr i} {
        set chcount 0
        set clist [lreplace $is(drmap) 0 0]
        foreach n $clist {
  	  if {![string compare $n $i]} {
	    set did [lindex $is(idmap) $i]
  	    set uplain [expr [lindex $ulist $i]&7]
	    set outgoing [expr [lindex $ulist $i]&128]
	    set usg($i,$chcount) [lindex $usage $uplain]
	    if [expr [lindex $flist $i] & (1<<$chcount)] {
	      set onl($i,$chcount) "Yes"
 	      if {$outgoing} {
	        set onlc($i,$chcount) red
	      } else {
	        set onlc($i,$chcount) yellow
	      }
	    } else {
	      set onl($i,$chcount) "No"
	      set onlc($i,$chcount) darkseagreen
	    }
	    set rem($i,$chcount) [lindex $rlist $i]
	    incr chcount
 	  }
        }
      }
    }
  } else {
    if {$active == 1} then {
      puts $gsock "240 Error No ISDN-infodevice found"
    }
  }
}

#-----------------------------------------------------------------------------
# upper part: retrieving info from /dev/isdninfo
#
# lower part: server part
#-----------------------------------------------------------------------------

# Implement the service
proc doService {sock msg} {
  global active chcount
    set msg [string toupper $msg]
    switch $msg {
	QUIT	{puts $sock "221 Goodbye."
		 set active 0
		 close $sock
		}	
	CHAN	{puts $sock "216-Sending channelcount."
#		 doloop
		 puts $sock "216-+$chcount"
		 puts $sock "216 End of channelcount."
		}
	STAT	{puts $sock "211-Sending status."
		 doloop
		 puts $sock "211"
		 puts $sock "211 End of status"
		}
	LIST	{puts $sock "210-Sending list."
		 doloop
		 puts $sock "210 End of LIST."
		}
	HELP	{puts $sock "214-List of available commands."
		 puts $sock "HELP	Get this Help"
		 puts $sock "LIST	List Info about ISDN-Channels"
		 puts $sock "QUIT	Close Connection"
		 puts $sock "STAT	Get server-status"
		 puts $sock "214 Direct Comments to m.bessler@gmx.net."
		}
	default {puts $sock "500 $msg: Command not recognized. Try HELP."}
    }
}

# Handles the input from the client and  client shutdown
proc  svcHandler {sock} {
  global active gsock
  set l [gets $sock]    ;# get the client packet
  if {[eof $sock]} {    ;# client gone or finished
     close $sock        ;# release the servers client channel
     set active 0
     set gsock 0
  } else {
    doService $sock $l
  }
}

# Accept-Connection handler for Server. 
# called When client makes a connection to the server
# Its passed the channel we're to communicate with the client on, 
# The address of the client and the port we're using
#
# Setup a handler for (incoming) communication on 
# the client channel - send connection Reply and log connection
proc accept {sock addr port} {
  
  # if {[badConnect $addr]} {
  #     close $sock
  #     return
  # }

  # Setup handler for future communication on client socket
  fileevent $sock readable [list svcHandler $sock]

  # Note we've accepted a connection (show how get peer info fm socket)
#  puts "Accept from [fconfigure $sock -peername]"

  # Read client input in lines, disable blocking I/O
  fconfigure $sock -buffering line -blocking 0

  # Send Acceptance string to client
  puts $sock "220 ISDNInfo-Sever ready. Welcome $addr:$port."
#  puts $sock "It is now [exec date]"
global gsock
set gsock $sock
  # log the connection
#  puts "Accepted connection from $addr at [exec date]"
}

global gsock
set gsock stdout
# Create a server socket on port $svcPort. 
# Call proc accept when a client attempts a connection.

socket -server accept $svcPort
main
vwait events    ;# handle events till variable events is set
